@@ -1,5 +1,5 @@
 #    Openvpn Applet
-#    Copyright (C) 2008  Mikko Vartiainen <mvartiainen@gmail.com>
+#    Copyright (C) 2008 - 2010  Mikko Vartiainen <mvartiainen@gmail.com>
 #
 #    This program is free software: you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 #    You should have received a copy of the GNU General Public License
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-enable_logging = False
+enable_logging = True
 
 if enable_logging:
     fd = open('/tmp/openvpn-applet.log','w+')
@@ -49,6 +49,7 @@ import re
 from UserDict import UserDict
 import socket
 import time
+import conic
 
 def copy_file(file):
     print "Copy: ", file
@@ -188,7 +189,7 @@ class vpn():
         return int(pid)
         
     def start(self,user=None,passwd=None,enable_log=False):
-        command =["sudo",self.startcmd,"--daemon","--writepid",self.pid_file,"--config",self.conf_file,"--management",self.socket_name,"unix"]
+        command =["sudo",self.startcmd,"--daemon","--writepid",self.pid_file,"--config",self.conf_file,"--persist-key", "--management",self.socket_name,"unix"]
         if enable_log:
             command.extend(['--verb','6'])
         log(command)
@@ -207,7 +208,11 @@ class vpn():
         
     def stop(self):
         log("stop")
-        self.__admin_command("signal SIGHUP\n")
+        self.__admin_command("signal SIGTERM\n")
+
+    def restart(self):
+        log("restart")
+        self.__admin_command("signal SIGUSR1\n")
 
     def get_conf_file(self):
         return self.conf_file        
@@ -283,6 +288,12 @@ class vpn_control():
         
     def stop(self, index):
         self.vpn_list[index].stop()
+
+    def restart_connected(self):
+        log("restart_connected")
+        for vpn in self.vpn_list:
+            if vpn.get_status():
+                vpn.restart()
         
     def status(self, index):
         if len(self.vpn_list) > 0:
@@ -315,7 +326,11 @@ class vpn_control():
 class openvpn_applet(hildondesktop.StatusMenuItem):
     def __init__(self):
         hildondesktop.StatusMenuItem.__init__(self)
-       
+
+        self.connection = conic.Connection()
+        self.connection.set_property("automatic-connection-events", True)
+        self.connection.connect("connection-event", self.connection_cb)
+
         self.vc = None
         self.banner = None
         self.timer = None
@@ -423,6 +438,23 @@ class openvpn_applet(hildondesktop.StatusMenuItem):
             print "open settings"
             self.open_settings(self)
 
+    def connection_cb(self, connection, event):
+        log("connection_cb",connection, event)
+        
+        status = event.get_status()
+        error = event.get_error()
+        iap_id = event.get_iap_id()
+        bearer = event.get_bearer_type()
+        
+        if status == conic.STATUS_CONNECTED:
+            log("CONNECTED",iap_id, bearer, status, error)
+            if self.vc != None:
+                self.vc.restart_connected()
+        elif status == conic.STATUS_DISCONNECTED:
+            log("(DISCONNECTED",iap_id, bearer, status, error)
+        elif status == conic.STATUS_DISCONNECTING:
+            log("(DISCONNECTING",iap_id, bearer, status, error)
+
 def user_pass_dlg(ask_passwd):
     if ask_passwd:
         dlg = hildon.LoginDialog(gtk.Window())
@@ -615,7 +647,7 @@ class OpenvpnAppletSettings(gtk.Dialog):
     def about_clicked_event(self,widget,data=None):
         from about import HeAboutDialog
 
-        HeAboutDialog.present(self,
+        response = HeAboutDialog.present(self,
                 about_name,
                 'openvpn-applet',
                 app_version,
@@ -625,6 +657,9 @@ class OpenvpnAppletSettings(gtk.Dialog):
                 about_bugtracker,
                 about_donate)
 
+        if response == HeAboutDialog.RESPONSE_WEBSITE or response == HeAboutDialog.RESPONSE_BUGTRACKER or response == HeAboutDialog.RESPONSE_DONATE:
+            self.destroy()
+
 
 class OpenvpnAppletTestConnection(gtk.Dialog):
     def __init__(self,title="Test Connection", parent=None, flags=0, buttons=None,vpn_conn=None):
@@ -677,26 +712,33 @@ class OpenvpnAppletTestConnection(gtk.Dialog):
         if self.timer == 'stop':
             return False
         if self.vpn_conn != None:
-            text = self.update_func()
-            self.textbuffer.set_text(text)
-            self.textview.scroll_to_iter(self.textbuffer.get_end_iter(),0.0,False,0,0)
+            new_text = self.update_func()
+            old_text = self.textbuffer.get_text(self.textbuffer.get_start_iter(),self.textbuffer.get_end_iter(),True)
+            if len(new_text) > len(old_text):
+                self.textbuffer.insert(self.textbuffer.get_end_iter(), new_text[len(old_text):])
+            elif len(new_text) < len(old_text):
+                self.textbuffer.set_text(new_text)
+                self.textview.scroll_to_iter(self.textbuffer.get_end_iter(),0.0,False,0,0)
         return True
     
     def log_button_clicked(self, widget, data=None):
         self.update_func = self.vpn_conn.get_log
+        self.textbuffer.set_text("")
         self.update_text()
 
     def status_button_clicked(self, widget, data=None):
         self.update_func = self.vpn_conn.get_statistics
+        self.textbuffer.set_text("")
         self.update_text()
 
     def state_button_clicked(self, widget, data=None):
         self.update_func = self.vpn_conn.get_state
+        self.textbuffer.set_text("")
         self.update_text()
 
     def save_button_clicked(self, widget, data=None):
         dlg = hildon.FileChooserDialog(self,gtk.FILE_CHOOSER_ACTION_SAVE,  hildon.FileSystemModel())
-        dlg.set_current_folder("/home/user/MyDocs")
+        dlg.set_current_folder(os.path.expanduser("~/MyDocs"))
         response = dlg.run()
         if response == gtk.RESPONSE_OK:
             filename = dlg.get_filename()
